
#import "NetServiceController.h"
#import "DotTouchConverter.h"
#import "IndexImage.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>


@implementation NetServiceController

@synthesize	delegate;
@synthesize	imageDoc;
@synthesize	isServiceStart;


-(void) startNetSercice
{
    uint16_t chosenPort = 0;
    
    if(!listeningSocket) {
        // Here, create the socket from traditional BSD socket calls, and then set up an NSFileHandle with that to listen for incoming connections.
		// ----------------------------------------------------------------------------------
        // ここで、伝統的なBSDソケット呼び出しからソケットを作成します、そして、
		// 次に、それでNSFileHandleをセットアップして、接続要求を聞こうとしてください。
		// ----------------------------------------------------------------------------------
		int fdForListening;
        struct sockaddr_in serverAddress;
        socklen_t namelen = sizeof(serverAddress);
		
        // In order to use NSFileHandle's acceptConnectionInBackgroundAndNotify method, we need to create a file descriptor that is itself a socket, bind that socket, and then set it up for listening. At this point, it's ready to be handed off to acceptConnectionInBackgroundAndNotify.
		// NSFileHandleのacceptConnectionInBackgroundAndNotifyメソッドを使用するために、
		// ----------------------------------------------------------------------------------
		// 私たちは、聴取にそれ自体でソケットであるファイルディスクリプタを作成するのが必要であり、そのソケットを縛って、
		// 次に、それを設定します。 ここに、それはacceptConnectionInBackgroundAndNotifyに渡される準備ができています。
		// ----------------------------------------------------------------------------------
        if((fdForListening = socket(AF_INET, SOCK_STREAM, 0)) > 0) {
            memset(&serverAddress, 0, sizeof(serverAddress));
            serverAddress.sin_family = AF_INET;
            serverAddress.sin_addr.s_addr = htonl(INADDR_ANY);
            serverAddress.sin_port = 0; // allows the kernel to choose the port for us.
			
            if(bind(fdForListening, (struct sockaddr *)&serverAddress, sizeof(serverAddress)) < 0) {
                close(fdForListening);
                return;
            }
			
            // Find out what port number was chosen for us.
			// ----------------------------------------------------------------------------------
			// どんなポートナンバーが私たちに選ばれたか調べてください。
			// ----------------------------------------------------------------------------------
            if(getsockname(fdForListening, (struct sockaddr *)&serverAddress, &namelen) < 0) {
                close(fdForListening);
                return;
            }
			
            chosenPort = ntohs(serverAddress.sin_port);
            
            if(listen(fdForListening, 1) == 0) {
                listeningSocket = [[NSFileHandle alloc] initWithFileDescriptor:fdForListening closeOnDealloc:YES];
            }
        }
    }
    
    if(!netService) {
        // lazily instantiate the NSNetService object that will advertise on our behalf.
		// ----------------------------------------------------------------------------------
		// 遅延は、当社に代わってアドバタイズしますNSNetServiceオブジェクトをインスタンス化します。
		// ----------------------------------------------------------------------------------
        netService = [[NSNetService alloc] initWithDomain:@"local" type:@"_DotTouch_WIFI_ID._tcp." name:@"" port:chosenPort];
        [netService setDelegate:self];
		
		if(netService)
			NSLog(@"netService Make!!");
    }
    
	//サービス開始
    if(netService && listeningSocket) {
        if( isServiceStart==NO ) {
            numberOfDownloads = 0;
            [[NSNotificationCenter defaultCenter] addObserver:self 
													 selector:@selector(connectionReceived:) 
														 name:NSFileHandleConnectionAcceptedNotification 
													   object:listeningSocket];
            [listeningSocket acceptConnectionInBackgroundAndNotify];
			[netService publish];
			isServiceStart = YES;
			NSLog(@"netService publish!!");
        }
    }
}

-(void) stopNetSercice
{
	//サービス停止
    if(netService && listeningSocket) {
		[netService stop];
		[[NSNotificationCenter defaultCenter] removeObserver:self 
														name:NSFileHandleConnectionAcceptedNotification 
													  object:listeningSocket];
		// There is at present no way to get an NSFileHandle to -stop- listening for events, so we'll just have to tear it down and recreate it the next time we need it.
		// ----------------------------------------------------------------------------------
		// ストップのイベントのため、我々はそれを切断し、それを我々に必要な次の時間を再作成する必要があります
		// リスニングへようこそ現在の方法でNSFileHandleを取得することです。
		// ----------------------------------------------------------------------------------
		[listeningSocket release];
		listeningSocket = nil;
		isServiceStart = NO;
	}
}




#pragma mark -
#pragma mark -- NSNetServiceDelegate --


// サービスの発行がうまく行った事をデリゲートに通知
- (void)netServiceDidPublish:(NSNetService *)sender
{
	NSLog(@"++ netServiceDidPublish: ++");
	
    if (self.delegate && [self.delegate respondsToSelector:@selector(netServiceDidEnableBonjour:withName:)])
		[self.delegate netServiceDidEnableBonjour:self withName:sender.name];
}

- (void)netService:(NSNetService *)sender didNotPublish:(NSDictionary *)errorDict {
	
	NSLog(@"++ netService: didNotPublish: ++");
	
	[super netService:sender didNotPublish:errorDict];
	
	[listeningSocket release];
	listeningSocket = nil;
	
	[netService release];
	netService = nil;
	
	if (self.delegate && [self.delegate respondsToSelector:@selector(netService:didNotEnableBonjour:)])
		[self.delegate netService:self didNotEnableBonjour:errorDict];
}

- (void)netServiceDidStop:(NSNetService *)sender {
	
	NSLog(@"++ netServiceDidStop: ++");
	
	// We'll need to release the NSNetService sending this, since we want to recreate it in sync with the socket at the other end. Since there's only the one NSNetService in this application, we can just release it.
	// ----------------------------------------------------------------------------------
	// 以来、私たちの同期の他の末尾にソケットを使用して再作成する我々は、NSNetServiceは、この送信をリリースする必要があります。
	// があるので、このアプリケーションで唯一の1 NSNetServiceのは、それを解放できます。
	// ----------------------------------------------------------------------------------
	[netService release];
	netService = nil;
}

// ----------------------------------------------------------------------------------
//アドレスを決定できなかった時に呼び出されます
// ----------------------------------------------------------------------------------
- (void)netService:(NSNetService *)sender didNotResolve:(NSDictionary *)errorDict {
	
	NSLog(@"++ netService: didNotResolve: ++");
	[netService release];
	netService = nil;
}

// ----------------------------------------------------------------------------------
//アドレスが決定された
// ----------------------------------------------------------------------------------
- (void)netServiceDidResolveAddress:(NSNetService *)service {
	NSLog(@"++ netServiceDidResolveAddress: ++");
}

#pragma mark -

// This object is also listening for notifications from its NSFileHandle.
// When an incoming connection is seen by the listeningSocket object, we get the NSFileHandle representing the near end of the connection. We write the thumbnail image to this NSFileHandle instance.
// ----------------------------------------------------------------------------------
// このオブジェクトは、そのNSFileHandleからの通知を聞いている。
// 着信接続をlisteningSocketオブジェクトが見られて、私たちはNSFileHandle接続の近端表現を取得します。
// 我々は、このNSFileHandleインスタンスへのサムネイルイメージを記述します。
// ----------------------------------------------------------------------------------
- (void)connectionReceived:(NSNotification *)aNotification {
	
	NSFileHandle * incomingConnection = [[aNotification userInfo] objectForKey:NSFileHandleNotificationFileHandleItem];
	
	IndexImage	*image = (IndexImage*)[imageDoc image];
	NSData		*imageData = [DotTouchConverter indexImageToDotTouchData:image];
	NSString*	name = [imageDoc fileName];
	NSData		*data = [DotTouchConverter dotTouchSendData:imageData Name:name];
	
	[[aNotification object] acceptConnectionInBackgroundAndNotify];
	[incomingConnection writeData:data];
	[incomingConnection closeFile];
	numberOfDownloads++;
}



@end
