
#import "WifiConnectController.h"


@implementation WifiConnectController

@synthesize	_services;
@synthesize	ownEntry;
@synthesize	currentResolve;
@synthesize	ownName;


- (id)init {
	
    self = [super init];
	if(self) {
		_connectMode = CONNECT_NON;
		_services = [[NSMutableArray alloc] init];
		selectTableIndex = -1;
	}
    return self;
}

-(void)awakeFromNib
{
	//サーバーサービス開始
	[self setupTCPServer];
	[cancelButton setTitle:@"キャンセル"];
	[self buttonEnableAndTitle];
	[table reloadData];
}


- (void) dealloc {
	
	if(_resvBuff)
		free(_resvBuff);
	[self destroyPicker];
	[_resvData release];
	[currentResolve stop];
	[currentResolve release];
	[super dealloc];
}


#pragma mark -




- (NSString *)ownName {
	return ownName;
}

- (void)setOwnName:(NSString *)name {
	if (ownName != name) {
		ownName = [name copy];
		if (self.ownEntry)
			[self._services addObject:self.ownEntry];
		
		NSNetService* service;
		for (service in _services) {
			if ([service.name isEqual:name]) {
				self.ownEntry = service;
				[_services removeObject:service];
				break;
			}
		}
		[table reloadData];
	}
}



#pragma mark -

// 接続／送信
- (IBAction)connectActionButtonClick:(id)sender
{
	if(_connectMode == CONNECT_NON){
		[connectActionButton setEnabled:NO];
		[self connectOKButtonClick:sender];
	}
	else if(_connectMode == CONNECT_WIFI_SEND){
		[self sendButtonClick:sender];
	}
}

//接続
- (IBAction)connectOKButtonClick:(id)sender
{
	if( [_services count] && selectTableIndex >= 0 ){
		
		if (self.currentResolve) {
			[self stopCurrentResolve];
		}
		
		//選択されたサービスのデリゲートを設定
		self.currentResolve = [_services objectAtIndex:selectTableIndex];
		[self.currentResolve setDelegate:self];
		[self.currentResolve resolveWithTimeout:0.0];
		
		[hostName release];
		hostName = [[self.currentResolve name] retain];
		_connectMode = CONNECT_WIFI_SEND;
	}
}

//送信
- (IBAction)sendButtonClick:(id)sender
{
	if([self sendMessage:SENDCODE_SEND]){
		[self destroyPicker];
		
		_services = [[NSMutableArray alloc] init];
		[self setupTCPServer];
		[self buttonEnableAndTitle];
		[table reloadData];
	}
}

//切断
- (IBAction)cancelButtonClick:(id)sender
{
	if( [self sendMessage:SENDCODE_CANCEL] ){
		//NSLog(@"-- キャンセルコード送信 --");
	}
	else {
	}
	_connectMode = 0;
	resv_code = 0;
	//[self setupTCPServer];
	[self buttonEnableAndTitle];
	[table reloadData];
	//[self destroyPicker];
}


//受信モード
- (IBAction)receivButtonClick:(id)sender
{
	//[self sendMessage:SENDCODE_SEND_MODE];
}



#pragma mark -

-(void) stopServer
{
	_bvc = nil;
	[_server release];
	_server = nil;
}


- (void)stopCurrentResolve {
	
	if(self.currentResolve){
		//NSLog(@"-- 現在のサービスを停止 --");
		[self.currentResolve stop];
		self.currentResolve = nil;
	}
}



-(BOOL) sendMessage:(int)code
{
	unsigned char	message;
	BOOL	ret = NO;
	resv_code = code;
	IndexImage	*image = (IndexImage*)[imageDoc image];
	//コード送信
	if(image==nil || code!=SENDCODE_SEND){
		if(image==nil)
			message = SENDCODE_CANCEL;
		else
			message = code;
		int				len = sizeof(unsigned char);
		if (m_outStream && [m_outStream hasSpaceAvailable]){
			len = [m_outStream write:&message maxLength:len];
			if(len != -1) {
				ret = YES;
				//NSLog([NSString stringWithFormat:@"-- コード送信:%d --", code]);
			}
		}
	}
	if( ret ) {
		if(_resvData)
			//NSLog(@"-- 受信データオブジェ解放 --");
		[_resvData release];
		_resvData = nil;
	}
	
	return	ret;
}

- (void) resvMessage
{
	unsigned int len = 0;
	len = [m_inStream read:streamTemp maxLength:STREAMTEMP_BYTES];
	
	if( !len ) {
		NSStreamStatus	state = [m_inStream streamStatus];
		if( state == NSStreamStatusAtEnd ){
			//NSLog(@"-- 受信　End --");
		}
		else{
			//NSLog(@"-- 受信　ERROR --");
		}
	} 
	else {
		if( len==sizeof(unsigned char) ){
			resv_code = streamTemp[0];
			// キャンセル
			if( resv_code==SENDCODE_CANCEL ) {
				//NSLog(@"-- キャンセル受信 --");
			}
			//接続確立
			else if(resv_code==SENDCODE_CONNECT_OK){
				//NSLog(@"接続確立");
			}
			//送信モード
			else if(resv_code==SENDCODE_SEND_MODE){
				//NSLog(@"送信モード");
			}
			//受信モード
			else if(resv_code==SENDCODE_RESV_MODE){
				//NSLog(@"受信モード");
			}
		}
		// データ受信
		else{
			resv_code = SENDCODE_SEND;
			[_resvData appendBytes:streamTemp length:len];
			//NSLog([NSString stringWithFormat:@"-- データ受信中(%d Bytes) --",len]);
		}
	}
}

-(void) setupNetworkConnectViewController:(int)type
{
	//受けて(Wi-Fi)
	if(_connectMode==CONNECT_WIFI_RECEIVE) {
		[_resvData release];
		_resvData = [[NSMutableData data] retain];
		//NSLog(@"-- 受信データオブジェを準備 --");
	}
	//送り手(Wi-Fi)
	if(_connectMode==CONNECT_WIFI_SEND) {
	}
}


- (void) buttonEnableAndTitle
{
	NSString	*str;
	
	NSString	*path;
	NSImage		*image=nil;
	
	//通常モード
	if(_connectMode == CONNECT_NON){
		[imageView setImage:image];
		//接続できるデバイスがある
		if( [_services count] && selectTableIndex!=-1){
			//接続
			str = [[NSBundle mainBundle] localizedStringForKey:@"connect" value:nil table:nil];
			[connectActionButton setTitle:str];
			if(imageDoc)
				[connectActionButton setEnabled:YES];
			else
				[connectActionButton setEnabled:NO];
			str = [[NSBundle mainBundle] localizedStringForKey:@"SelectDevice" value:nil table:nil];
		}
		//接続できるデバイスがない
		else{
			//接続
			str = [[NSBundle mainBundle] localizedStringForKey:@"connect" value:nil table:nil];
			[connectActionButton setTitle:str];
			[connectActionButton setEnabled:NO];
			str = [[NSBundle mainBundle] localizedStringForKey:@"SelectWifiInMenu" value:nil table:nil];
		}
		[cancelButton setEnabled:NO];
	}
	//送信モード
	if(_connectMode == CONNECT_WIFI_SEND){
		path = [[NSBundle mainBundle] pathForResource:@"wifi_send" ofType:@"gif"];
		image = [[NSImage alloc] initWithContentsOfFile:path];
		[imageView setImage:image];
		//送信
		str = [[NSBundle mainBundle] localizedStringForKey:@"send" value:nil table:nil];
		[connectActionButton setTitle:str];
		[connectActionButton setEnabled:YES];
		[cancelButton setEnabled:YES];
		str = [NSString stringWithFormat:@"%@%@",hostName, [[NSBundle mainBundle] localizedStringForKey:@"ToSend" value:nil table:nil] ];
		//str = [[NSBundle mainBundle] localizedStringForKey:@"SendImage" value:nil table:nil];
	}
	//受信モード
	if(_connectMode == CONNECT_WIFI_RECEIVE){
		path = [[NSBundle mainBundle] pathForResource:@"wifi_resv" ofType:@"gif"];
		image = [[NSImage alloc] initWithContentsOfFile:path];
		[imageView setImage:image];
		//受信待機
		[connectActionButton setEnabled:NO];
		[cancelButton setEnabled:YES];
		str = [[NSBundle mainBundle] localizedStringForKey:@"ResiveWaiting" value:nil table:nil];
	}
	[deviceName setStringValue:str];
}


#pragma mark -
#pragma mark -- TCPServer --

- (void) setupTCPServer
{
	_connectMode = CONNECT_NON;
	
	[self stopServer];
	
	[m_inStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[m_inStream release];
	m_inStream = nil;
	_inReady = NO;
	
	[m_outStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[m_outStream release];
	m_outStream = nil;
	_outReady = NO;
	
	_server = [[TCPServer alloc] init];
	[_server setDelegate:self];
	NSError *error;
	if(_server == nil || ![_server start:&error]) {
		return;
	}
	
	if(![_server enableBonjourWithDomain:@"local" applicationProtocol:[TCPServer bonjourTypeFromIdentifier:kWiFiID] name:nil]) {
		return;
	}
}

- (void) openStreams
{
	//NSLog(@"-- ストリームオープン --");
	
	m_inStream.delegate = self;
	[m_inStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[m_inStream open];
	
	m_outStream.delegate = self;
	[m_outStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[m_outStream open];
}

- (void) presentPicker:(NSString *)name {
	
	//NSLog(@"-- presentPicker: --");
	
	[self stopCurrentResolve];
	//[_netServiceBrowser stop];
	[_services removeAllObjects];
	
	if (!_netServiceBrowser) {
		_netServiceBrowser = [[NSNetServiceBrowser alloc] init];
		[_netServiceBrowser setDelegate:self];
		[_netServiceBrowser searchForServicesOfType:[TCPServer bonjourTypeFromIdentifier:kWiFiID] inDomain:@"local"];
		//NSLog(@"-- ネットサービスブラウザー起動 --");
	}
	self.ownName = name;
	[table reloadData];
	//NSLog([NSString stringWithFormat:@"-- オーナーネーム設定(%@) --",name]);
}

- (void) destroyPicker
{
	//NSLog(@"-- 全サービスを破棄 --");
	
//	[_netServiceCnt stopNetSercice];
//	[_netServiceCnt release];
//	_netServiceCnt = nil;
	
	[self stopServer];
	
	NSNetService* service;
	for (service in _services) {
		[service stop];
	}
	[table reloadData];
	[_services release];
	_services = nil;
	selectTableIndex = -1;
	
	[m_inStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
	[m_inStream release];
	m_inStream = nil;
	
	[m_outStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
	[m_outStream release];
	m_outStream = nil;
	
	[ownName release];
	ownName = nil;
	
	[hostName release];
	hostName = nil;
	
	[ownEntry stop];
	[ownEntry release];
	ownEntry = nil;
	
	[_netServiceBrowser stop];
	[_netServiceBrowser release];
	_netServiceBrowser = nil;
}


#pragma mark -- TCPServerDelegate --

- (void) serverDidEnableBonjour:(TCPServer *)server withName:(NSString *)string
{
	[self presentPicker:string];
}

//接続された
- (void)didAcceptConnectionForServer:(TCPServer *)server inputStream:(NSInputStream *)istr outputStream:(NSOutputStream *)ostr
{
	if (m_inStream || m_outStream || server != _server)
		return;
	
	[self stopServer];
	
	m_inStream = istr;
	[m_inStream retain];
	
	m_outStream = ostr;
	[m_outStream retain];
	
	[self openStreams];
	
}

/*
#pragma mark -
#pragma mark -- NetServiceControllerDelegate --

- (void) netService:(NetServiceController*)server didNotEnableBonjour:(NSDictionary *)errorDict
{
	//NSLog(@"-- NetServiceControllerDelegate server:server didNotEnableBonjour: --");
}
- (void) netServiceDidEnableBonjour:(NetServiceController *)server withName:(NSString *)string
{
	[self presentPicker:string];
}

//接続された
- (void)didAcceptConnectionForService:(NetServiceController *)server inputStream:(NSInputStream *)istr outputStream:(NSOutputStream *)ostr
{
	if (m_inStream || m_outStream )
		return;
	
	[_netServiceCnt stopNetSercice];
	[_netServiceCnt release];
	_netServiceCnt = nil;
	
	m_inStream = istr;
	[m_inStream retain];
	
	m_outStream = ostr;
	[m_outStream retain];
	
	[self openStreams];
	
	if(_connectMode==CONNECT_NON){
		_connectMode = CONNECT_WIFI_RECEIVE;
		NSString	*str = [[NSBundle mainBundle] localizedStringForKey:@"ResiveWaiting" value:nil table:nil];
		[deviceName setStringValue:str];
		[table reloadData];
	}
	
}
*/


#pragma mark -
#pragma mark -- NetServiceBrowserDelegate --

//ネットサービスが消える
- (void)netServiceBrowser:(NSNetServiceBrowser *)netServiceBrowser didRemoveService:(NSNetService *)service moreComing:(BOOL)moreComing {
	
	//NSLog([NSString stringWithFormat:@"-- ネットサービスが消える(%@) --",[service name]]);
	
	if (self.currentResolve && [service isEqual:self.currentResolve]) {
		[self stopCurrentResolve];
	}
	[_services removeObject:service];
	if (self.ownEntry == service)
		self.ownEntry = nil;
	
	[self buttonEnableAndTitle];
	[table reloadData];
}	

//ネットサービスが見つかる
- (void)netServiceBrowser:(NSNetServiceBrowser *)netServiceBrowser didFindService:(NSNetService *)service moreComing:(BOOL)moreComing {
	
	if ([service.name isEqual:self.ownName]){
		self.ownEntry = service;
		//NSLog(@"-- ネットサービスが見つかる(自分) --");
	}
	else{
		[_services addObject:service];
		//NSLog([NSString stringWithFormat:@"-- ネットサービスが見つかる(%@) --",[service name]]);
	}
	
	if (!moreComing) {
	}
	
	[table reloadData];
	if(_connectMode==CONNECT_NON)
		[self buttonEnableAndTitle];
}


#pragma mark -
#pragma mark -- NSNetServiceDelegate --

//アドレスが決定された
- (void)netServiceDidResolveAddress:(NSNetService *)service {
	
	self.currentResolve = nil;
	if (!service) {
		[self setupTCPServer];
		return;
	}
	
	if (![service getInputStream:&m_inStream outputStream:&m_outStream]) {
		return;
	}
	
	[service resolveWithTimeout:0.0];
	[self openStreams];
	
	[self buttonEnableAndTitle];
	[table reloadData];
}




#pragma mark -
#pragma mark -- NSStreamDelegate --


- (void)stream:(NSStream *)aStream handleEvent:(NSStreamEvent)event
{
	NSStreamEvent	evCode = event;
	switch(evCode)
	{
		case NSStreamEventOpenCompleted:
			
			if (aStream == m_inStream){
				_inReady = YES;
				//NSLog(@"-- ストリームオープン(Input) --");
			}
			else{
				_outReady = YES;
				//NSLog(@"-- ストリームオープン(Output) --");
			}
			
			if (_inReady && _outReady) {
				[self setupNetworkConnectViewController:NETWORKTYPE_WIFI];
			}
			break;
			
		case NSStreamEventHasBytesAvailable:
			if (aStream == m_inStream) {
				[self resvMessage];	// 受信中
			}
			break;
			
		case NSStreamEventErrorOccurred:
			break;
			
		case NSStreamEventEndEncountered:
			//受信完了
			if (aStream == m_inStream ) {
				//NSLog(@"-- 受信完了 --");
				if(resv_code==SENDCODE_SEND){
					//ファイル名を取り出す
					unsigned char	*nameBytes = (unsigned char*)[_resvData bytes];
					int				len = nameBytes[0];
					NSString		*name = [[NSString alloc] initWithBytes:(nameBytes+1) length:len encoding:NSUTF8StringEncoding];
					// 受信したデータをDotShot形式に変換
					IndexImage*	indexImage = [DotTouchConverter dotTouchDataToIndexImage:_resvData];
					if(indexImage){
						NSDictionary	*dic = [NSDictionary dictionaryWithObject:name forKey:@"fileName"];
						[[NSNotificationCenter defaultCenter] postNotificationName:@"indexImagePostNotification" object:indexImage userInfo:dic];
					}
				}
			}
			
			if(_resvData){
				//NSLog(@"-- 受信データオブジェ解放 --");
				[_resvData release];
				_resvData = nil;
			}
			resv_code = SENDCODE_CANCEL;
			_connectMode = 0;
			
			//NSLog(@"-- ストリーム解放 --");
			[m_inStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
			[m_inStream release];
			m_inStream = nil;
			_inReady = NO;
			
			[m_outStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
			[m_outStream release];
			m_outStream = nil;
			_outReady = NO;
			
			[self buttonEnableAndTitle];
			[table reloadData];
			break;
	}
}

#pragma mark -
#pragma mark -- テーブルビューのデータソース --

//行の数を返す
- (int)numberOfRowsInTableView:(NSTableView*)tableView
{
	int count = [_services count];
	if(count==0)
		selectTableIndex = -1;
	
	return count;
}
//テーブルの行項目を返す
- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)tableColumn row:(int)row
{
    id				identifier;
	NSString*		name = nil;
    
	identifier = [tableColumn identifier];
	
	NSNetService *service = [_services objectAtIndex:row];
	//名前
	if ([identifier isEqualToString:@"name"]){
		name =[service name];
	}
	
	return name;
}


//選択変更
-(void) tableViewSelectionDidChange:(NSNotification *)aNotification
{
	id				obj =[aNotification object];
	int				row =[obj selectedRow];
	if( row != -1 ){
		selectTableIndex = row;
	}
	
	[self buttonEnableAndTitle];
}
//テーブルビューの選択許可
- (BOOL)tableView:(NSTableView *)aTableView shouldSelectRow:(int)rowIndex
{
	return	YES;
}



@end
