
#import "NetWorkController.h"


@implementation NetWorkController

@synthesize delegate;


#pragma mark -

-(id) initWithDelegate:(id)obj
{
	self = [super init];
	if(self){
		self.delegate = obj;
	}
	return	self;
}


- (void)dealloc
{
	[self stop];
	[super dealloc];
}


-(void) start
{
	[self setupTCPServer];
}

-(void) stop
{
	if(_server != nil) {
		[self destroyPicker];
	}
	[_resvData release];
	_resvData = nil;
}

-(void) stopServer
{
	_bvc = nil;
	[_server release];
	_server = nil;
}


//データーの送信
-(BOOL) sendData:(NSData*)data
{
	BOOL	ret = NO;
	if(_server != nil) {
		unsigned char	*message = (unsigned char*)[data bytes];
		int				len = [data length];
		if (_outStream && [_outStream hasSpaceAvailable]){
			if([_outStream write:message maxLength:len] != -1)
				ret = YES;
		}
	}
	
	return	ret;
}







#pragma mark -
#pragma mark -- TCPServer --

//サーバーサービスを作成
- (void) setupTCPServer
{
	[self stopServer];
	
	[_inStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[_inStream release];
	_inStream = nil;
	_inReady = NO;
	
	[_outStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[_outStream release];
	_outStream = nil;
	_outReady = NO;
	
	_server = [TCPServer new];
	[_server setDelegate:self];
	NSError *error;
	if(_server == nil || ![_server start:&error]) {
		return;
	}
	
	if(![_server enableBonjourWithDomain:@"local" applicationProtocol:[TCPServer bonjourTypeFromIdentifier:kWiFiID] name:nil]) {
		return;
	}
}


//ストリームを開く
- (void) openStreams
{
	_inStream.delegate = self;
	[_inStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[_inStream open];

	_outStream.delegate = self;
	[_outStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[_outStream open];
}

//ブラウザーの準備
- (void) presentPicker:(NSString *)name {
	
	if( _bvc==nil ){
		_bvc = [[BrowserViewController alloc] initWithTitle:@"wifi" showDisclosureIndicators:NO showCancelButton:NO];
		[_bvc searchForServicesOfType:[TCPServer bonjourTypeFromIdentifier:kWiFiID] inDomain:@"local"];
		[_bvc setDelegate:self];
		[_bvc setOwnName:name];
		[_bvc autorelease];
		
		if (self.delegate && [self.delegate respondsToSelector:@selector(netWorkControllerDidSetupBrowserViewController:BrowserController:)])
			[self.delegate netWorkControllerDidSetupBrowserViewController:self BrowserController:_bvc];
		
		_bvc = nil;
	}
}

//全てを破棄
- (void) destroyPicker
{
	[self stopServer];
	
	[_inStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
	[_inStream release];
	_inStream = nil;
	
	[_outStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
	[_outStream release];
	_outStream = nil;
	
}

#pragma mark -- TCPServerDelegate --

//ネットサービスが有効
- (void) serverDidEnableBonjour:(TCPServer *)server withName:(NSString *)string
{
	[self presentPicker:string];
}

//接続された
- (void)didAcceptConnectionForServer:(TCPServer *)server inputStream:(NSInputStream *)istr outputStream:(NSOutputStream *)ostr
{
	if (_inStream || _outStream || server != _server)
		return;
	
	[self stopServer];
	
	_inStream = istr;
	[_inStream retain];
	
	_outStream = ostr;
	[_outStream retain];
	
	[self openStreams];
}




#pragma mark -
#pragma mark -- BrowserViewControllerDelegate --

//接続した
- (void) browserViewController:(BrowserViewController *)bvc didResolveInstance:(NSNetService *)netService
{
	if (!netService) {
		[self setupTCPServer];
		return;
	}
	
	if (![netService getInputStream:&_inStream outputStream:&_outStream]) {
		return;
	}
	
	[self openStreams];
}


#pragma mark -
#pragma mark -- NSStreamDelegate --

- (void) stream:(NSStream *)stream handleEvent:(NSStreamEvent)eventCode
{
	unsigned int len = 0;
	
	switch(eventCode) {
		case NSStreamEventOpenCompleted:
			
			[self stopServer];
			
			if (stream == _inStream)
				_inReady = YES;
			else
				_outReady = YES;
			
			if (_inReady && _outReady) {
				if (self.delegate && [self.delegate respondsToSelector:@selector(netWorkControllerConnect:)])
					[self.delegate netWorkControllerConnect:self];
			}
			break;
			
		case NSStreamEventHasBytesAvailable:
			if (stream == _inStream) {
				// 受信中
				len = [_inStream read:streamTemp maxLength:256];
				if( !len ) {
					NSStreamStatus	state = [_inStream streamStatus];
					if( state == NSStreamStatusAtEnd )
						NSLog(@"End");
					else
						NSLog(@"ERROR");
				} 
				else {
					// データ受信
					if( _resvData )
						_resvData = [[NSMutableData data] retain];
					[_resvData appendBytes:streamTemp length:len];
				}
			}
			break;
			
		case NSStreamEventErrorOccurred:
			break;
			
		case NSStreamEventEndEncountered:
			//受信完了
			if (stream == _inStream ) {
				if (self.delegate && [self.delegate respondsToSelector:@selector(netWorkControllerReceive:Data:)]){
					[self.delegate netWorkControllerReceive:self Data:_resvData];
					[_resvData setLength:0];
				}
			}
			break;
	}
}


@end
